@@ -1,33 +1,31 @@
-# -*- coding: utf-8 -*-
 """The main xonsh script."""
-import os
-import sys
-import enum
 import argparse
 import builtins
 import contextlib
+import enum
+import os
 import signal
+import sys
 import traceback
 
+import xonsh.procs.pipelines as xpp
 from xonsh import __version__
-from xonsh.timings import setup_timings
-from xonsh.lazyasd import lazyobject
-from xonsh.shell import Shell
-from xonsh.pretty import pretty
+from xonsh.built_ins import XSH
+from xonsh.codecache import run_code_with_cache, run_script_with_cache
+from xonsh.environ import make_args_env, xonshrc_context
+from xonsh.events import events
 from xonsh.execer import Execer
-from xonsh.proc import HiddenCommandPipeline
+from xonsh.imphooks import install_import_hooks
 from xonsh.jobs import ignore_sigtstp
-from xonsh.tools import print_color, to_bool_or_int
+from xonsh.lazyasd import lazyobject
+from xonsh.lazyimps import pyghooks, pygments
 from xonsh.platform import HAS_PYGMENTS, ON_WINDOWS
-from xonsh.codecache import run_script_with_cache, run_code_with_cache
+from xonsh.pretty import pretty
+from xonsh.shell import Shell
+from xonsh.timings import setup_timings
+from xonsh.tools import display_error_message, print_color, to_bool_or_int
 from xonsh.xonfig import print_welcome_screen
 from xonsh.xontribs import xontribs_load
-from xonsh.lazyimps import pygments, pyghooks
-from xonsh.imphooks import install_import_hooks
-from xonsh.events import events
-from xonsh.environ import xonshrc_context, make_args_env
-from xonsh.built_ins import XonshSession, load_builtins
-
 
 events.transmogrify("on_post_init", "LoadEvent")
 events.doc(
@@ -107,14 +105,14 @@ def get_setproctitle():
 
 
 def path_argument(s):
-    """Return a path only if the path is actually legal
+    """Return a path only if the path is actually legal (file or directory)
 
     This is very similar to argparse.FileType, except that it doesn't return
     an open file handle, but rather simply validates the path."""
 
     s = os.path.abspath(os.path.expanduser(s))
-    if not os.path.isfile(s):
-        msg = "{0!r} must be a valid path to a file".format(s)
+    if not os.path.exists(s):
+        msg = f"{s!r} must be a valid path to a file or directory"
         raise argparse.ArgumentTypeError(msg)
     return s
 
@@ -128,19 +126,18 @@ def parser():
         dest="help",
         action="store_true",
         default=False,
-        help="show help and exit",
+        help="Show help and exit.",
     )
     p.add_argument(
         "-V",
         "--version",
-        dest="version",
-        action="store_true",
-        default=False,
-        help="show version information and exit",
+        action="version",
+        help="Show version information and exit.",
+        version=f"xonsh/{__version__}",
     )
     p.add_argument(
         "-c",
-        help="Run a single command and exit",
+        help="Run a single command and exit.",
         dest="command",
         required=False,
         default=None,
@@ -148,7 +145,7 @@ def parser():
     p.add_argument(
         "-i",
         "--interactive",
-        help="force running in interactive mode",
+        help="Force running in interactive mode.",
         dest="force_interactive",
         action="store_true",
         default=False,
@@ -156,23 +153,15 @@ def parser():
     p.add_argument(
         "-l",
         "--login",
-        help="run as a login shell",
+        help="Run as a login shell.",
         dest="login",
         action="store_true",
         default=False,
     )
-    p.add_argument(
-        "--config-path",
-        help="DEPRECATED: static configuration files may now be used "
-        "in the XONSHRC file list, see the --rc option.",
-        dest="config_path",
-        default=None,
-        type=path_argument,
-    )
     p.add_argument(
         "--rc",
         help="The xonshrc files to load, these may be either xonsh "
-        "files or JSON-based static configuration files.",
+        "files or directories containing xonsh files",
         dest="rc",
         nargs="+",
         type=path_argument,
@@ -180,21 +169,21 @@ def parser():
     )
     p.add_argument(
         "--no-rc",
-        help="Do not load the .xonshrc files",
+        help="Do not load any xonsh RC files.",
         dest="norc",
         action="store_true",
         default=False,
     )
     p.add_argument(
         "--no-script-cache",
-        help="Do not cache scripts as they are run",
+        help="Do not cache scripts as they are run.",
         dest="scriptcache",
         action="store_false",
         default=True,
     )
     p.add_argument(
         "--cache-everything",
-        help="Use a cache, even for interactive commands",
+        help="Use a cache, even for interactive commands.",
         dest="cacheall",
         action="store_true",
         default=False,
@@ -202,7 +191,7 @@ def parser():
     p.add_argument(
         "-D",
         dest="defines",
-        help="define an environment variable, in the form of "
+        help="Define an environment variable, in the form of "
         "-DNAME=VAL. May be used many times.",
         metavar="ITEM",
         action="append",
@@ -211,8 +200,10 @@ def parser():
     p.add_argument(
         "--shell-type",
         help="What kind of shell should be used. "
-        "Possible options: readline, prompt_toolkit, random. "
-        "Warning! If set this overrides $SHELL_TYPE variable.",
+        "Possible options: "
+        + ", ".join(Shell.shell_type_aliases.keys())
+        + ". Warning! If set this overrides $SHELL_TYPE variable.",
+        metavar="SHELL_TYPE",
         dest="shell_type",
         choices=tuple(Shell.shell_type_aliases.keys()),
         default=None,
@@ -229,14 +220,14 @@ def parser():
     p.add_argument(
         "file",
         metavar="script-file",
-        help="If present, execute the script in script-file" " and exit",
+        help="If present, execute the script in script-file and exit.",
         nargs="?",
         default=None,
     )
     p.add_argument(
         "args",
         metavar="args",
-        help="Additional arguments to the script specified " "by script-file",
+        help="Additional arguments to the script specified by script-file.",
         nargs=argparse.REMAINDER,
         default=[],
     )
@@ -247,13 +238,15 @@ def _pprint_displayhook(value):
     if value is None:
         return
     builtins._ = None  # Set '_' to None to avoid recursion
-    if isinstance(value, HiddenCommandPipeline):
+    if isinstance(value, xpp.HiddenCommandPipeline):
         builtins._ = value
         return
-    env = builtins.__xonsh__.env
+    env = XSH.env
+    printed_val = None
     if env.get("PRETTY_PRINT_RESULTS"):
         printed_val = pretty(value)
-    else:
+    if not isinstance(printed_val, str):
+        # pretty may fail (i.e for unittest.mock.Mock)
         printed_val = repr(value)
     if HAS_PYGMENTS and env.get("COLOR_RESULTS"):
         tokens = list(pygments.lex(printed_val, lexer=pyghooks.XonshLexer()))
@@ -271,39 +264,62 @@ class XonshMode(enum.Enum):
     interactive = 3
 
 
-def start_services(shell_kwargs, args):
+def start_services(shell_kwargs, args, pre_env=None):
     """Starts up the essential services in the proper order.
     This returns the environment instance as a convenience.
     """
-    install_import_hooks()
+    if pre_env is None:
+        pre_env = {}
     # create execer, which loads builtins
     ctx = shell_kwargs.get("ctx", {})
     debug = to_bool_or_int(os.getenv("XONSH_DEBUG", "0"))
     events.on_timingprobe.fire(name="pre_execer_init")
     execer = Execer(
-        xonsh_ctx=ctx,
+        filename="<stdin>",
         debug_level=debug,
         scriptcache=shell_kwargs.get("scriptcache", True),
         cacheall=shell_kwargs.get("cacheall", False),
     )
+    XSH.load(ctx=ctx, execer=execer)
     events.on_timingprobe.fire(name="post_execer_init")
+
+    install_import_hooks(execer)
+
     # load rc files
     login = shell_kwargs.get("login", True)
-    env = builtins.__xonsh__.env
-    rc = shell_kwargs.get("rc", None)
-    rc = env.get("XONSHRC") if rc is None else rc
-    if (
+    rc_cli = shell_kwargs.get("rc")
+    env = XSH.env
+    for k, v in pre_env.items():
+        env[k] = v
+
+    # determine which RC files to load, including whether any RC directories
+    # should be scanned for such files
+    if shell_kwargs.get("norc") or (
         args.mode != XonshMode.interactive
         and not args.force_interactive
         and not args.login
     ):
-        #  Don't load xonshrc if not interactive shell
-        rc = None
+        # if --no-rc was passed, or we're not in an interactive shell and
+        # interactive mode was not forced, then disable loading RC files and dirs
+        rc = ()
+        rcd = ()
+    elif rc_cli:
+        # if an explicit --rc was passed, then we should load only that RC
+        # file, and nothing else (ignore both XONSHRC and XONSHRC_DIR)
+        rc = [r for r in rc_cli if os.path.isfile(r)]
+        rcd = [r for r in rc_cli if os.path.isdir(r)]
+    else:
+        # otherwise, get the RC files from XONSHRC, and RC dirs from XONSHRC_DIR
+        rc = env.get("XONSHRC")
+        rcd = env.get("XONSHRC_DIR")
+
     events.on_pre_rc.fire()
-    xonshrc_context(rcfiles=rc, execer=execer, ctx=ctx, env=env, login=login)
+    XSH.rc_files = xonshrc_context(
+        rcfiles=rc, rcdirs=rcd, execer=execer, ctx=ctx, env=env, login=login
+    )
     events.on_post_rc.fire()
     # create shell
-    builtins.__xonsh__.shell = Shell(execer=execer, **shell_kwargs)
+    XSH.shell = Shell(execer=execer, **shell_kwargs)
     ctx["__name__"] = "__main__"
     return env
 
@@ -312,7 +328,6 @@ def premain(argv=None):
     """Setup for main xonsh entry point. Returns parsed arguments."""
     if argv is None:
         argv = sys.argv[1:]
-    builtins.__xonsh__ = XonshSession()
     setup_timings(argv)
     setproctitle = get_setproctitle()
     if setproctitle is not None:
@@ -321,26 +336,22 @@ def premain(argv=None):
     if args.help:
         parser.print_help()
         parser.exit()
-    if args.version:
-        version = "/".join(("xonsh", __version__))
-        print(version)
-        parser.exit()
     shell_kwargs = {
         "shell_type": args.shell_type,
         "completer": False,
         "login": False,
         "scriptcache": args.scriptcache,
         "cacheall": args.cacheall,
-        "ctx": builtins.__xonsh__.ctx,
+        "ctx": XSH.ctx,
     }
     if args.login or sys.argv[0].startswith("-"):
         args.login = True
         shell_kwargs["login"] = True
     if args.norc:
-        shell_kwargs["rc"] = ()
+        shell_kwargs["norc"] = True
     elif args.rc:
         shell_kwargs["rc"] = args.rc
-    setattr(sys, "displayhook", _pprint_displayhook)
+    sys.displayhook = _pprint_displayhook
     if args.command is not None:
         args.mode = XonshMode.single_command
         shell_kwargs["shell_type"] = "none"
@@ -354,13 +365,15 @@ def premain(argv=None):
         args.mode = XonshMode.interactive
         shell_kwargs["completer"] = True
         shell_kwargs["login"] = True
-    env = start_services(shell_kwargs, args)
-    env["XONSH_LOGIN"] = shell_kwargs["login"]
+
+    pre_env = {
+        "XONSH_LOGIN": shell_kwargs["login"],
+        "XONSH_INTERACTIVE": args.force_interactive
+        or (args.mode == XonshMode.interactive),
+    }
+    env = start_services(shell_kwargs, args, pre_env=pre_env)
     if args.defines is not None:
         env.update([x.split("=", 1) for x in args.defines])
-    env["XONSH_INTERACTIVE"] = args.force_interactive or (
-        args.mode == XonshMode.interactive
-    )
     return args
 
 
@@ -369,30 +382,47 @@ def _failback_to_other_shells(args, err):
     # as an interactive one for safe.
     if hasattr(args, "mode") and args.mode != XonshMode.interactive:
         raise err
+
     foreign_shell = None
-    shells_file = "/etc/shells"
-    if not os.path.exists(shells_file):
-        # right now, it will always break here on Windows
-        raise err
-    excluded_list = ["xonsh", "screen"]
-    with open(shells_file) as f:
-        for line in f:
-            line = line.strip()
-            if not line or line.startswith("#"):
-                continue
-            if "/" not in line:
-                continue
-            _, shell = line.rsplit("/", 1)
-            if shell in excluded_list:
-                continue
-            if not os.path.exists(line):
-                continue
-            foreign_shell = line
-            break
+
+    # look first in users login shell $SHELL.
+    # use real os.environ, in case Xonsh hasn't initialized yet
+    # but don't fail back to same shell that just failed.
+
+    try:
+        env_shell = os.getenv("SHELL")
+        if env_shell and os.path.exists(env_shell) and env_shell != sys.argv[0]:
+            foreign_shell = env_shell
+    except Exception:
+        pass
+
+    # otherwise, find acceptable shell from (unix) list of installed shells.
+
+    if not foreign_shell:
+        excluded_list = ["xonsh", "screen"]
+        shells_file = "/etc/shells"
+        if not os.path.exists(shells_file):
+            # right now, it will always break here on Windows
+            raise err
+        with open(shells_file) as f:
+            for line in f:
+                line = line.strip()
+                if not line or line.startswith("#"):
+                    continue
+                if "/" not in line:
+                    continue
+                _, shell = line.rsplit("/", 1)
+                if shell in excluded_list:
+                    continue
+                if not os.path.exists(line):
+                    continue
+                foreign_shell = line
+                break
+
     if foreign_shell:
         traceback.print_exc()
         print("Xonsh encountered an issue during launch", file=sys.stderr)
-        print("Failback to {}".format(foreign_shell), file=sys.stderr)
+        print(f"Failback to {foreign_shell}", file=sys.stderr)
         os.execlp(foreign_shell, foreign_shell)
     else:
         raise err
@@ -402,7 +432,7 @@ def main(argv=None):
     args = None
     try:
         args = premain(argv)
-        return main_xonsh(args)
+        sys.exit(main_xonsh(args))
     except Exception as err:
         _failback_to_other_shells(args, err)
 
@@ -418,15 +448,30 @@ def main_xonsh(args):
         signal.signal(signal.SIGTTOU, func_sig_ttin_ttou)
 
     events.on_post_init.fire()
-    env = builtins.__xonsh__.env
-    shell = builtins.__xonsh__.shell
+    env = XSH.env
+    shell = XSH.shell
+    history = XSH.history
+    exit_code = 0
+
+    if shell and not env["XONSH_INTERACTIVE"]:
+        shell.ctx.update({"exit": sys.exit})
+
+    # store a sys.exc_info() tuple to record any exception that might occur in the user code that we are about to execute
+    # if this does not change, no exceptions were thrown. Otherwise, print a traceback that does not expose xonsh internals
+    exc_info = None, None, None
+
     try:
         if args.mode == XonshMode.interactive:
             # enter the shell
+
+            # Setted again here because it is possible to call main_xonsh() without calling premain(), namely in the tests.
             env["XONSH_INTERACTIVE"] = True
+
             ignore_sigtstp()
-            if env["XONSH_INTERACTIVE"] and not any(
-                os.path.isfile(i) for i in env["XONSHRC"]
+            if (
+                env["XONSH_INTERACTIVE"]
+                and not any(os.path.isfile(i) for i in env["XONSHRC"])
+                and not any(os.path.isdir(i) for i in env["XONSHRC_DIR"])
             ):
                 print_welcome_screen()
             events.on_pre_cmdloop.fire()
@@ -436,7 +481,15 @@ def main_xonsh(args):
                 events.on_post_cmdloop.fire()
         elif args.mode == XonshMode.single_command:
             # run a single command and exit
-            run_code_with_cache(args.command.lstrip(), shell.execer, mode="single")
+            exc_info = run_code_with_cache(
+                args.command.lstrip(),
+                "<string>",
+                shell.execer,
+                glb=shell.ctx,
+                mode="single",
+            )
+            if history is not None and history.last_cmd_rtn is not None:
+                exit_code = history.last_cmd_rtn
         elif args.mode == XonshMode.script_from_file:
             # run a script contained in a file
             path = os.path.abspath(os.path.expanduser(args.file))
@@ -445,27 +498,44 @@ def main_xonsh(args):
                 env.update(make_args_env())  # $ARGS is not sys.argv
                 env["XONSH_SOURCE"] = path
                 shell.ctx.update({"__file__": args.file, "__name__": "__main__"})
-                run_script_with_cache(
+                exc_info = run_script_with_cache(
                     args.file, shell.execer, glb=shell.ctx, loc=None, mode="exec"
                 )
             else:
-                print("xonsh: {0}: No such file or directory.".format(args.file))
-                events.on_exit.fire()
-                sys.exit(1)
+                print(f"xonsh: {args.file}: No such file or directory.")
+                exit_code = 1
         elif args.mode == XonshMode.script_from_stdin:
             # run a script given on stdin
             code = sys.stdin.read()
-            run_code_with_cache(
-                code, shell.execer, glb=shell.ctx, loc=None, mode="exec"
+            exc_info = run_code_with_cache(
+                code, "<stdin>", shell.execer, glb=shell.ctx, loc=None, mode="exec"
             )
+    except SyntaxError:
+        exit_code = 1
+        debug_level = env.get("XONSH_DEBUG", 0)
+        if debug_level == 0:
+            # print error without tracktrace
+            display_error_message(sys.exc_info())
+        else:
+            # pass error to finally clause
+            exc_info = sys.exc_info()
     finally:
+        if exc_info != (None, None, None):
+            err_type, err, _ = exc_info
+            if err_type is SystemExit:
+                raise err
+            else:
+                traceback.print_exception(*exc_info)
+                exit_code = 1
         events.on_exit.fire()
-    postmain(args)
+        postmain(args)
+    return exit_code
 
 
 def postmain(args=None):
     """Teardown for main xonsh entry point, accepts parsed arguments."""
-    builtins.__xonsh__.shell = None
+    XSH.unload()
+    XSH.shell = None
 
 
 @contextlib.contextmanager
@@ -475,7 +545,7 @@ def main_context(argv=None):
     up the shell.
     """
     args = premain(argv)
-    yield builtins.__xonsh__.shell
+    yield XSH.shell
     postmain(args)
 
 
@@ -519,14 +589,13 @@ def setup(
     ctx = {} if ctx is None else ctx
     # setup xonsh ctx and execer
     if not hasattr(builtins, "__xonsh__"):
-        execer = Execer(xonsh_ctx=ctx)
-        builtins.__xonsh__ = XonshSession(ctx=ctx, execer=execer)
-        load_builtins(ctx=ctx, execer=execer)
-        builtins.__xonsh__.shell = Shell(execer, ctx=ctx, shell_type=shell_type)
-    builtins.__xonsh__.env.update(env)
-    install_import_hooks()
-    builtins.aliases.update(aliases)
+        execer = Execer(filename="<stdin>")
+        XSH.load(ctx=ctx, execer=execer)
+        XSH.shell = Shell(execer, ctx=ctx, shell_type=shell_type)
+    XSH.env.update(env)
+    install_import_hooks(XSH.execer)
+    XSH.aliases.update(aliases)
     if xontribs:
         xontribs_load(xontribs)
-    tp = builtins.__xonsh__.commands_cache.threadable_predictors
+    tp = XSH.commands_cache.threadable_predictors
     tp.update(threadable_predictors)
