@@ -1,71 +1,63 @@
-# -*- coding: utf-8 -*-
 """Hooks for pygments syntax highlighting."""
 import os
 import re
-import sys
-import builtins
 import stat
+import sys
+import typing as tp
 from collections import ChainMap
 from collections.abc import MutableMapping
 from keyword import iskeyword
-import typing as tp
 
-from xonsh.built_ins import XSH
-from xonsh.lazyimps import os_listxattr
-
-from pygments.lexer import inherit, bygroups, include
+import pygments.util
+from pygments.lexer import bygroups, include, inherit
 from pygments.lexers.agile import Python3Lexer
+from pygments.style import Style
 from pygments.token import (
+    Comment,
+    Error,
+    Generic,
     Keyword,
     Name,
-    Comment,
-    String,
-    Error,
     Number,
     Operator,
-    Generic,
-    Whitespace,
-    Token,
     Punctuation,
+    String,
     Text,
+    Token,
+    Whitespace,
     _TokenType,
 )
-from pygments.style import Style
-import pygments.util
-
-from xonsh.commands_cache import CommandsCache
-from xonsh.lazyasd import LazyObject, LazyDict, lazyobject
-from xonsh.tools import (
-    ON_WINDOWS,
-    intensify_colors_for_cmd_exe,
-    ANSICOLOR_NAMES_MAP,
-    PTK_NEW_OLD_COLOR_MAP,
-    hardcode_colors_for_win10,
-    FORMATTER,
-)
 
+from xonsh.built_ins import XSH
 from xonsh.color_tools import (
-    RE_BACKGROUND,
     BASE_XONSH_COLORS,
+    RE_BACKGROUND,
     RE_XONSH_COLOR,
-    make_palette,
     find_closest_color,
     iscolor,
+    make_palette,
     warn_deprecated_no_color,
 )
-from xonsh.style_tools import norm_name, DEFAULT_STYLE_DICT
-from xonsh.lazyimps import terminal256, html
+from xonsh.commands_cache import CommandsCache
+from xonsh.events import events
+from xonsh.lazyasd import LazyDict, LazyObject, lazyobject
+from xonsh.lazyimps import html, os_listxattr, terminal256
 from xonsh.platform import (
     os_environ,
-    win_ansi_support,
     ptk_version_info,
     pygments_version_info,
+    win_ansi_support,
+)
+from xonsh.pygments_cache import add_custom_style, get_style_by_name
+from xonsh.style_tools import DEFAULT_STYLE_DICT, norm_name
+from xonsh.tools import (
+    ANSICOLOR_NAMES_MAP,
+    FORMATTER,
+    ON_WINDOWS,
+    PTK_NEW_OLD_COLOR_MAP,
+    hardcode_colors_for_win10,
+    intensify_colors_for_cmd_exe,
 )
-
-from xonsh.pygments_cache import get_style_by_name, add_custom_style
-
-from xonsh.events import events
-
 
 #
 # Colors and Styles
@@ -170,7 +162,7 @@ def color_name_to_pygments_code(name, styles):
         return styles[token]
     m = RE_XONSH_COLOR.match(name)
     if m is None:
-        raise ValueError("{!r} is not a color!".format(name))
+        raise ValueError(f"{name!r} is not a color!")
     parts = m.groupdict()
     # convert regex match into actual pygments colors
     if parts["reset"] is not None:
@@ -245,18 +237,21 @@ def color_token_by_name(xc: tuple, styles=None) -> _TokenType:
         try:
             styles = XSH.shell.shell.styler.styles  # type:ignore
         except AttributeError:
-            return Color
+            pass
 
     tokName = xc[0]
-    pc = color_name_to_pygments_code(xc[0], styles)
+    if styles:
+        pc = color_name_to_pygments_code(xc[0], styles)
 
-    if len(xc) > 1:
-        pc += " " + color_name_to_pygments_code(xc[1], styles)
-        tokName += "__" + xc[1]
+        if len(xc) > 1:
+            pc += " " + color_name_to_pygments_code(xc[1], styles)
+            tokName += "__" + xc[1]
 
     token = getattr(Color, norm_name(tokName))
-    if token not in styles or not styles[token]:
+
+    if styles and (token not in styles or not styles[token]):
         styles[token] = pc
+
     return token
 
 
@@ -266,7 +261,7 @@ def partial_color_tokenize(template):
     These sub-strings maybe templates themselves.
     """
     if XSH.shell is not None:
-        styles = __xonsh__.shell.shell.styler.styles
+        styles = XSH.shell.shell.styler.styles
     else:
         styles = None
     color = Color.DEFAULT
@@ -387,7 +382,7 @@ class XonshStyle(Style):
                 pygments_style_by_name(value)
             except Exception:
                 print(
-                    "Could not find style {0!r}, using default".format(value),
+                    f"Could not find style {value!r}, using default",
                     file=sys.stderr,
                 )
                 value = "default"
@@ -1649,16 +1644,12 @@ class XonshLexer(Python3Lexer):
     def __init__(self, *args, **kwargs):
         # If the lexer is loaded as a pygment plugin, we have to mock
         # __xonsh__.env and __xonsh__.commands_cache
-        if not hasattr(builtins, "__xonsh__"):
-            from argparse import Namespace
-
-            builtins.__xonsh__ = Namespace()
         if not hasattr(XSH, "env"):
             XSH.env = {}
             if ON_WINDOWS:
                 pathext = os_environ.get("PATHEXT", [".EXE", ".BAT", ".CMD"])
                 XSH.env["PATHEXT"] = pathext.split(os.pathsep)
-        if not getattr(XSH, "commands_cache", None):
+        if getattr(XSH, "commands_cache", None) is None:
             XSH.commands_cache = CommandsCache()
         _ = XSH.commands_cache.all_commands  # NOQA
         super().__init__(*args, **kwargs)
@@ -1735,7 +1726,7 @@ class XonshLexer(Python3Lexer):
         with root or subproc state"""
         start = 0
         state = ("root",)
-        m = re.match(r"(\s*)({})".format(COMMAND_TOKEN_RE), text)
+        m = re.match(rf"(\s*)({COMMAND_TOKEN_RE})", text)
         if m is not None:
             yield m.start(1), Whitespace, m.group(1)
             start = m.end(1)
